@@ -1,7 +1,8 @@
 from abc import abstractmethod, ABC
 from .apis import upload_file
+from validators import url
+from urllib.parse import quote
 import json
-
 class Output(ABC):
     type: str
 
@@ -19,7 +20,7 @@ class TextOutput(Output):
             'type': self.type,
             'message': self.message,
         }
-
+    
 class ImageOutput(Output):
     type = 'image-output'
     def __init__(self, image_str: str, subtitle: str = ""):
@@ -32,7 +33,7 @@ class ImageOutput(Output):
             'image_str': self.image_str,
             'subtitle': self.subtitle,
         }
-
+    
 class LinkOutput(Output):
     type = 'link-output'
     def __init__(self, link_url: str, link_text: str = "Click here"):
@@ -58,7 +59,7 @@ class FileOutput(Output):
             'message': self.file if isinstance(self.file, str) else upload_file(self.file),
             'downloadText': self.download_text,
         }
-
+    
 class HTMLOutput(Output):
     type = 'html-output'
     def __init__(self, html):
@@ -69,7 +70,7 @@ class HTMLOutput(Output):
             'type': self.type,
             'html': self.html
         }
-
+    
 class PandasOutput(Output):
     type = 'pandas-output'
     def __init__(self, df):
@@ -90,4 +91,23 @@ class PlotlyOutput(Output):
         return {
             'type': self.type,
             'figure': json.loads(self.fig.to_json())
-        }
\ No newline at end of file
+        }
+    
+class IFrameOutput(Output):
+    type = 'iframe-output'
+    def __init__(self, url_or_html, width, height):
+        if url(url_or_html):
+            self.url = url_or_html
+        else:
+            self.url = f"data:text/html,{quote(url_or_html)}" 
+        
+        self.width = width
+        self.height = height
+    
+    def json(self):
+        return {
+            'type': self.type,
+            'url': self.url,
+            'width': self.width,
+            'height': self.height,
+        }
